// $RCSfile: AbstractArgumentAtom.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.argument;


/**
 * Every Atom must inherit from this class. Its main function is to
 * provide the standard implemention of {@link #equals}, {@link #matches}
 * and {@link #hashCode}.
 *
 * @version $Revision: 1.3 $
 * @author  Michael Meyling
 */
public abstract class AbstractArgumentAtom
        extends AbstractArgument implements Argument {


    /**
     * @inherited
     * @return always 0
     */
    public int getArgumentSize() {
        return 0;
    }


    /**
     * @inherited
     * @param   i   number of argument
     * @return  will never return anything, this is an Atom
     * @throws  IllegalArgumentException will always be thrown
     */
    public Argument getArgument(int i)
            throws IllegalArgumentException {
        throw new IllegalArgumentException(
            ArgumentConstants.NO_ARGUMENT_NUMBER);
    }


    /** @inherited */
    public abstract boolean equals(Object object);


    /** @inherited */
    public abstract Argument copy();


    /** @inherited */
    public final Argument create(final Argument[] arguments) {
        throw new IllegalArgumentException("Programming error");    // TODO
    }


    /** @inherited */
    public abstract String toString();

}
