// $RCSfile: Counter.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.argument;


/**
 * An object of this class represents a non negative integer.
 *
 * @version $Revision: 1.3 $
 * @author  Michael Meyling
 */
public class Counter extends AbstractArgumentAtom implements Argument,
        CounterMarker {

    /** the plain number */
    private final int number;


    /**
     * Constructs a <code>Number</code>.
     *
     * @param   number  non negative integer (0 is ok)
     * @throws  ArgumentException    if number < 0
     */
    public Counter(final int number) throws ArgumentException {
        if (number < 0) {
            throw new ArgumentException(
                ArgumentConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ArgumentConstants.NO_NEGATIVE_INTEGER);
        }
        this.number = number;
    }


    /**
     * Get the number.
     *
     * @return  number
     */
    public final int getNumber() {
        return number;
    }


    /** @inherited */
    public final boolean equals(final Object object) {
        if (object instanceof Counter) {
            return ((Counter) object).number == number;
        } else {
            return false;
        }
    }


    /** @inherited */
    public final Argument copy() {
        return this;
        // TODO deep copy???
//        return new Counter(number);
    }


    /** @inherited */
    public  final String toString() {
        return Integer.toString(number);
    }


}
