// $RCSfile: FoundException.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.argument;


/**
 * Helps constructing a parent list.
 *
 *
 * @version $Revision: 1.3 $
 * @author  Michael Meyling
 */
public class FoundException extends Exception {


    /** parent children list */
    private final VariableList list;


    /**
     * Constructs an argument exception, starts with an argument.
     *
     * @param  argument this argument is the deepest node
     */
    public FoundException(final Argument argument) {
        this.list = new VariableList( new Argument[] {} );
        this.list.add(argument);
    }


    /**
     * Constructs an argument exception, starts with an argument.
     *
     * @param   argument this argument is added as an parent
     * @throws  FoundException  throws itself again
     */
    public final void addAndThrow(final Argument argument)
            throws FoundException {
        this.list.add(argument);
        throw this;
    }


    /**
     * Get current parent children list.
     *
     * @return  parent children list
     */
    public final VariableList getList() {
        return this.list;
    }

    /**
     * Get last argument entry.
     *
     * @return  last argument
     */
    public final Argument getLastEntry() {
        return this.list.getArgument(list.getArgumentSize() - 1);
    }


}
