// $RCSfile: Text.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.argument;


import com.meyling.principia.io.Utility;


/**
 * An object of this class represents a text string.
 *
 * @version $Revision: 1.3 $
 * @author  Michael Meyling
 */
public class Text extends AbstractArgumentAtom implements Argument {

    /** the plain text */
    private final String text;


    /**
     * Constructs a <code>Text</code>.
     *
     * @param   text    text string
     * @throws  IllegalArgumentException    if <code>text</code> was a nullpointer
     */
    public Text(final String text) {
        if (text == null) {
            throw new IllegalArgumentException(
                ArgumentConstants.NULLPOINTER_PARAMETER);
        }
        this.text = text;
    }


    /**
     * Get the text.
     *
     * @return  text
     */
    public final String getText() {
        return text;
    }


    /** @inherited */
    public final boolean equals(Object object) {
        if (object instanceof Text) {
            return ((Text) object).text.equals(text);
        } else {
            return false;
        }
    }


    /** @inherited */
    // TODO deep copy?
    public final Argument copy() {
        return this;
//        return new Text(text);
    }


    /** @inherited */
    public final String toString() {
        return Utility.quote(text);
    }


}
