// $RCSfile: ParsingException.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.io;


/**
 * This is a construction (and checking) exception for arguments.
 *
 * @version $Revision: 1.3 $
 * @author  Michael Meyling
 */
public class ParsingException extends Exception {


    /** exception code */
    private final int code;

    /** description of reason for exception */
    private final String description;

    /** source position of problem */
    private final int row;

    /** source position of problem */
    private final int column;

    /** problematic line */
    private final String line;

    /** address of problematic module */
    private final String address;


    /**
     * Constructs an argument exception.
     *
     * @param  code         exception cathegory
     * @param  description  what is the problem
     * @param  row          row number of problem
     * @param  column       column number of problem
     * @param  line         problematic line
     * @param  address      address that describes the module location
     */
    public ParsingException(final int code, final String description,
            final int row, final int column, final String line,
            final String address) {
        super(description);
        this.code = code;
        this.description = description;
        this.row = row;
        this.column = column;
        this.line = line;
        this.address = address;
    }


    /**
     * Get exception code.
     *
     * @return  exception code
     */
    public final int getCode() {
        return this.code;
    }


    /**
     * Get exception code.
     *
     * @return  exception description
     */
    public final String getDescription() {
        return this.description;
    }


    /**
     * Get row of problem.
     *
     * @return  problem row position)
     */
    public final int getRow() {
        return this.row;
    }


    /**
     * Get problem position.
     *
     * @return  problem position)
     */
    public final int getColumn() {
        return this.column;
    }


    /**
     * Get problematic source line.
     *
     * @return  source line
     */
    public final String getLine() {
        return this.line;
    }


    /**
     * Get address of problematic module.
     *
     * @return  address of problematic module
     */
    public final String getAddress() {
        return this.address;
    }


}
