// $RCSfile: BasicPatternVariables.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.*;
import com.meyling.principia.logic.basic.*;

import java.util.Map;
import java.util.Iterator;


/**
 * Set of pattern variables.
 *
 * @version $Revision: 1.5 $
 * @author  Michael Meyling
 */
public class BasicPatternVariables extends PatternVariables{


    /**
     * Construct an empty set of pattern variables.
     */
    public BasicPatternVariables() {
        super();
    }


    /**
     * Construct an set of pattern variables by an PatternVariables object.
     *
     * @param var   PatternVariables to copy from
     */
    public BasicPatternVariables(final PatternVariables var) {
        super(var);
    }


    /**
     * Fill contents with subject variables, starting with
     * given number + 1.
     *
     * @param   greater   the created subject variables will
     *                    start with <code>greater + 1</code>
     * @throws  IllegalArgumentException    if there is already
     *           a value assigned in any pattern variable
     *           or if creation of Counter failed
     */
    public final void fillWithSubjectVariables(final int greater)
            throws IllegalArgumentException {
        try {
            int i = 1;
            for (Iterator iterator = getMap().entrySet().iterator();
                    iterator.hasNext(); ) {
                final Map.Entry entry = (Map.Entry) iterator.next();
                ((PatternVariable) entry.getValue()).setContent(
                    new SubjectVariable(new Argument[] {
                        new Counter(greater + i++)}));
            }
        } catch (ArgumentException e) {     // TODO: really convert???
            throw new IllegalArgumentException(e.getMessage());
        }
    }


    /**
     * Get highest number of each entry.
     *
     * @return  maxiumum of number value for all entries,
     *          if getSize() == 0 it returns -1
     */
    public final int getHighestNumber() {
        int maximum = -1;   // minimum number value is 0
        for (Iterator iterator = getMap().entrySet().iterator();
                iterator.hasNext(); ) {
            Map.Entry entry = (Map.Entry) iterator.next();
            final int number = ((Counter) ((Argument) entry.getValue())
                .getArgument(0)).getNumber();
            if (maximum < number) {
                maximum = number;
            }
        }
        return maximum;
    }


    /**
     * Get VariableList of all variables, the list is orderd by type and
     * number of the pattern variables.
     *
     * @throws  IllegalArgumentException if there is a programming error.
     */
    public final VariableList getVariables()
            throws IllegalArgumentException {
        try {
            return new VariableList(
                (Argument[]) getMap().values().toArray(new Argument[0]));
        } catch (ArrayStoreException e) {     // TODO: really convert???
            throw new IllegalArgumentException(e.getMessage());
        }
    }



}
