// $RCSfile: BasicSubjectVariablePatternVariable.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.*;
import com.meyling.principia.logic.basic.*;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;


/**
 * Pattern variable, for matching purposes.
 *
 * @version $Revision: 1.4 $
 * @author  Michael Meyling
 */
public class BasicSubjectVariablePatternVariable extends PatternVariable
        implements SubjectVariableMarker {

    /** number of pattern variable */
    private final Counter number;

    /** assignment of pattern variable */
    private Argument argument = null;


    /**
     * Constructs a pattern variable.
     *
     * @param   arguments  first argument is number of this pattern
     *          variable
     * @throws  ArgumentException    if there is not exactly one
     *          argument that is an instance of {@link Counter}
     */
    public BasicSubjectVariablePatternVariable(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (arguments.length != 1) {
            if (arguments.length == 0) {
                throw new ArgumentException(
                    ArgumentConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                    ArgumentConstants.PATTERN_VARIABLE
                    + ArgumentConstants.OPERATOR_WITH_ONE_ARGUMENT);
            } else {
                throw new ArgumentException(
                    ArgumentConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                    ArgumentConstants.PATTERN_VARIABLE
                    + ArgumentConstants.OPERATOR_WITH_ONE_ARGUMENT,
                    arguments[arguments.length - 1],
                    false);
            }
        }
        if (!(arguments[0] instanceof Counter)) {
            throw new ArgumentException(
                ArgumentConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ArgumentConstants.PATTERN_VARIABLE
                + ArgumentConstants.FIRST_ARGUMENT_TYPE
                + ArgumentConstants.COUNTER,
                arguments[0]);
        }
        this.number = (Counter) arguments[0];
    }

    /**
     * Constructs a pattern variable.
     *
     * @param   number of this pattern variable
     * @throws  ArgumentException    if there couldn't be made a
     *          {@link Counter} out of <code>number</code>
     */
    public BasicSubjectVariablePatternVariable(final int number)
            throws ArgumentException {
        this(new Argument[] { new Counter(number) });
    }



    /** @inherited */
    public final PatternVariables getPatternVariables() {
        BasicPatternVariables variables = new BasicPatternVariables();
        variables.add(this);
        return variables;
    }


    /**
     * Is the argument matching the pattern variable?
     * Here true is returned if argument is an SubjectVariable.
     *
     * @param   argument    argument to match pattern variable
     * @return  <code>true</code>
     */
    public boolean isMatching(final Argument argument) {
        if (argument == null) {
            return false;
        }
        return argument instanceof SubjectVariable;
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new BasicSubjectVariablePatternVariable(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "@bx" + number.toString();
    }

}
