// $RCSfile: Disjunction.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.*;


/**
 * Disjunction of formulas. Implementation of the locigal &quot;or&quot;.
 *
 * @version $Revision: 1.4 $
 * @author  Michael Meyling
 */
public class Disjunction extends LogicalOperatorWithTwoFormulas {


    /**
     * Constructs a logical disjunction.
     *
     * @param   arguments     formulas to operate on
     * @throws  ArgumentException    if there are not exactly two
     *          arguments that are instances of {@link Formula} or
     *          if the is a subject variable that would exist
     *          bound and unbound in the resulting formula
     *
     */
    public Disjunction(final Argument[] arguments)
            throws ArgumentException {
        super (arguments);
    }


    /**
     * Constructs a logical disjunction.
     *
     * @param   formula1    first formula
     * @param   formula2    second formula
     * @throws  ArgumentException    if the is a subject variable
     *          that would exist bound and unbound in the resulting formula
     *
     */
    public Disjunction(final Formula formula1, final Formula formula2)
            throws ArgumentException {
        super(formula1, formula2);
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new Disjunction(arguments);
    }


    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("(");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append(" v ");
            }
            buffer.append(getArgument(i));
        }
        buffer.append(")");
        return buffer.toString();
    }


}
