// $RCSfile: Formula.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.*;


/**
 * This interface tells us about the methods a Formula must have.
 * @version $Revision: 1.3 $
 * @author  Michael Meyling
 */
public interface Formula extends Argument {

    /**
     * Get the number of part formulas that this formula contains.
     * @return number of part formulas that this formula is build of
     */
    public abstract int getPartFormulaSize();


    /**
     * Get the requested part formula
     * @param   i   number of part formula (starting with <code>0</code>)
     * @return  <code>i</code>-th part formula
     * @throws  IllegalArgumentException    if <code>i</code> is not inbetween
     *           <code>0</code> and <code>{@link #getPartFormulaSize} - 1</code>
     */
    public abstract Formula getPartFormula(int i) throws IllegalArgumentException;


    /**
     * Get all free subject variables.
     * @return  all free subject variables
     */
    public abstract SubjectVariables getFreeSubjectVariables();


    /**
     * Get all bounded subject variables.
     * @return  all bounded subject variables
     */
    public abstract SubjectVariables getBoundSubjectVariables();


    /**
     *  Get all subject variables that occur in this formula.
     *  @return  all subject variables
     */
    public abstract SubjectVariables getSubjectVariables();


    /**
     * Replace the <code>occurrence</code>-th occurence of a bound
     * {@link SubjectVariable} by another one.
     *
     * @param   counter         number of matches so far
     * @param   occurrence      number of matches to act on
     * @param   search          subject variable to look for
     * @param   replacement     replacement subject variable
     * @return  replaced formula
     * @throws  ArgumentException   if creating replacement failed
     */
    public abstract Formula replaceBoundSubjectVariable(Enumerator counter,
            int occurrence, SubjectVariable search,
            SubjectVariable replacement) throws ArgumentException;


    /**
     * Replace every bound SubjectVariable by a
     * BasicSubjectVariablePatternVariable
     *
     * @param   counter contains starting number for
     *     replacement pattern variable, is increased
     *     after one complete replacement
     * @throws  ArgumentException
     */
    public abstract Formula replaceBoundSubjectVariables(
        Enumerator counter)
        throws ArgumentException;



}
