// $RCSfile: Implication.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.*;


/**
 * Implication of formulas. Implementation of the locigal &quot;if&quot;.
 *
 * @version $Revision: 1.4 $
 * @author  Michael Meyling
 */
public class Implication extends LogicalOperatorWithTwoFormulas {


    /**
     * Constructs a logical implication.
     *
     * @param   arguments     formulas to operate on
     * @throws  ArgumentException    if there are not exactly two
     *          arguments that are instances of {@link Formula} or
     *          if the is a subject variable that would exist
     *          bound and unbound in the resulting formula
     *
     */
    public Implication(final Argument[] arguments) throws ArgumentException {
        super (arguments);
    }


    /**
     * Constructs a logical implication.
     *
     * @param   formula1    first formula
     * @param   formula2    second formula
     * @throws  ArgumentException    if the is a subject variable
     *          that would exist bound and unbound in the resulting formula
     *
     */
    public Implication(final Formula formula1, final Formula formula2)
            throws ArgumentException {
        super(formula1, formula2);
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new Implication(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "(" + getArgument(0).toString() + " => "
            + getArgument(1).toString() + ")";
    }


}
