// $RCSfile: PropositionVariable.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.*;


/**
 * Represents an arbitrary logical proposition.
 *
 * @version $Revision: 1.4 $
 * @author  Michael Meyling
 */
public class PropositionVariable extends AbstractFormula
        implements Argument, Formula {

    /** number of proposition variable */
    private CounterMarker number;


    /**
     * Constructs a proposition variable.
     *
     * @param   arguments  first argument is number of this proposition
     *          variable
     * @throws  ArgumentException    if there is not exactly one
     *          argument that is an instance of {@link Counter}
     */
    public PropositionVariable(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (arguments.length != 1) {
            throw new ArgumentException(
                BasicConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                BasicCreator.getName(PropositionVariable.class)
                + BasicConstants.OPERATOR_WITH_ONE_ARGUMENT);
        }
        if (!(arguments[0] instanceof CounterMarker)) {
            throw new ArgumentException(
                BasicConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                BasicCreator.getName(PropositionVariable.class)
                + BasicConstants.FIRST_ARGUMENT_TYPE
                + BasicConstants.COUNTER_OR_PATTERN,
                arguments[0]);
        }
        this.number = (CounterMarker) arguments[0];
    }


    /**
     * Constructs certain proposition variable.
     *
     * @param   number  number of proposition variable
     * @throws  ArgumentException    if Counter couldn't be created
     */
    public PropositionVariable(final int number)
            throws ArgumentException {
        this(new Argument[] { new Counter(number) } );
    }


    /**
     * Get the identity number of this proposition variable.
     *
     * @return  identity number
     */
    public int getNumber() {
        return number.getNumber();
    }


    /**
     * @inherited
     * @return  always 0
     */
    public final int getPartFormulaSize() {
        return 0;
    }


    /**
     * @inherited
     * @return  will never occur
     * @throw   IllegalArgumentException    always thrown
     */
    public final Formula getPartFormula(final int i) {
        throw new IllegalArgumentException(
            BasicConstants.NO_ARGUMENT_NUMBER);
    }


    /** @inherited */
    public final SubjectVariables getFreeSubjectVariables() {
        return new SubjectVariables();
    };


    /** @inherited */
    public final SubjectVariables getBoundSubjectVariables() {
        return new SubjectVariables();
    }


    /** @inherited */
    public final SubjectVariables getSubjectVariables() {
        return new SubjectVariables();
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new PropositionVariable(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "A" + number.toString();
    }


}
