// $RCSfile: SubjectVariable.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.basic;


import com.meyling.principia.argument.*;


/**
 * Subject variable, represents a subject of the universe of discourse.
 *
 * @version $Revision: 1.4 $
 * @author  Michael Meyling
 */
public class SubjectVariable extends AbstractArgumentList
        implements SubjectVariableMarker {


    /** the number of the subject variable */
    private CounterMarker number;


    /**
     * Constructs certain subject variable.
     *
     * @param   arguments  first argument is number of subject variable
     * @throws  ArgumentException    if there is not exactly one
     *          argument that is an instance of {@link CounterMarker}
     */
    public SubjectVariable(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (arguments.length != 1) {
            throw new ArgumentException(
                BasicConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                BasicCreator.getName(SubjectVariable.class)
                + BasicConstants.OPERATOR_WITH_ONE_ARGUMENT);
        }
        if (!(arguments[0] instanceof CounterMarker)) {
            throw new ArgumentException(
                BasicConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                BasicCreator.getName(SubjectVariable.class)
                + BasicConstants.FIRST_ARGUMENT_TYPE
                + BasicConstants.COUNTER,
                arguments[0]);
        }
        this.number = (CounterMarker) arguments[0];
    }


    /**
     * Constructs certain subject variable.
     *
     * @param   number  number of subject variable
     * @throws  ArgumentException    if Counter couldn't be created
     */
    public SubjectVariable(final int number)
            throws ArgumentException {
        this(new Argument[] { new Counter(number) } );
    }


    /**
     * Get the number of the subject variable.
     *
     * @return  number of subject variable
     */
    public final int getNumber() {
        return number.getNumber();
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new SubjectVariable(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "x" + number.toString();
    }


}
