// $RCSfile: Axiom.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.paragraph;


import com.meyling.principia.argument.*;
import com.meyling.principia.module.*;
import com.meyling.principia.logic.basic.*;


/**
 * Logical Axiom.
 * @version $Revision: 1.3 $
 * @author  Michael Meyling
 */
public class Axiom extends AbstractArgumentList implements Argument,
        ParagraphCheck {


    /**
     * Constructs a logical Axiom.
     *
     * @param   arguments     formula to be an Axiom
     * @throws  ArgumentException    if there is not exactly one
     *          argument or the first argument is not an instance
     *          of {@link Formula} or it contains any
     *          {@link PatternVariable}.
     */
    public Axiom(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);

        if (arguments.length != 1) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.OPERATOR_WITH_ONE_ARGUMENT);
        }
        if (!(arguments[0] instanceof Formula)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.FIRST_ARGUMENT_TYPE
                + ParagraphConstants.FORMULA,
                arguments[0]);
        }
        if (arguments[0].containsPatternVariables()) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_ARGUMENT_CHECK_FAILED,
                ParagraphConstants.NO_PATTERN_VARIABLE_IN_AXIOM,
                arguments[0]);
        }
    }


    /**
     * Tries to add this axiom.
     *
     * @param   module   module
     * @param   label    label of paragraph
     */
    public final void check(final Module module, final String label) {
        module.addAxiom(label, this, module);
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new Axiom(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "Axiom " + getArgument(0).toString();
    }

}
