// $RCSfile: LinkLabel.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.paragraph;


import com.meyling.principia.argument.*;
import com.meyling.principia.logic.basic.*;
import com.meyling.principia.module.*;


/**
 * A reference label.
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public class LinkLabel extends AbstractArgumentList
        implements Argument, Labeled, Textable {


    /** label */
    private final String label;


    /**
     * Constructs a reference label.
     *
     * @param   arguments     {@link Text} to be a label
     * @throws  ArgumentException    if there is not exactly one
     *          argument or the first argument is not an instance
     *          of {@link Text} or it isn't build of lowercase letters
     *          and digits and its first character is a lowercase letter.
     */
    public LinkLabel(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);

        if (arguments.length != 1) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ParagraphCreator.getName(this.getClass())
                + ParagraphConstants.OPERATOR_WITH_ONE_ARGUMENT);
        }
        if (!(arguments[0] instanceof Text)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ParagraphCreator.getName(this.getClass())
                + ParagraphConstants.FIRST_ARGUMENT_TYPE
                + ParagraphConstants.QUOTED_STRING,
                arguments[0]);
        }
        label = ((Text) arguments[0]).getText();
        if (label.length() == 0) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ParagraphCreator.getName(this.getClass())
                + ParagraphConstants.NO_EMPTY_STRING,
                arguments[0]);
        }
        if (!label.toLowerCase().equals(label)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ParagraphCreator.getName(this.getClass())
                + ParagraphConstants.ONLY_LOWER_CASE_CHARS,
                arguments[0]);
        }
        if (!Character.isLetter(label.charAt(0))) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ParagraphCreator.getName(this.getClass())
                + ParagraphConstants.MUST_START_WITH_LETTER,
                arguments[0]);
        }
        for (int i = 1; i < label.length(); i++) {
            if (!Character.isLetterOrDigit(label.charAt(i))) {
                throw new ArgumentException(
                    ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                    ParagraphCreator.getName(this.getClass())
                    + ParagraphConstants.MUST_BE_LETTER_DIGIT_STRING,
                    arguments[0]);
            }
        }
    }


    /** @inherited */
    public final String getLabel() {
        return label;
    }


    /** @inherited */
    public final String getText() {
        return ((Text) getArgument(0)).getText();
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new LinkLabel(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "Label " + getArgument(0).toString();
    }

}
