// $RCSfile: ProofLine.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.paragraph;


import com.meyling.principia.argument.*;
import com.meyling.principia.module.*;
import com.meyling.principia.logic.basic.*;
import com.meyling.principia.logic.rule.*;


/**
 * A proof line of a mathematical proof.
 *
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public class ProofLine extends AbstractArgumentList
        implements Argument {

    /**
     * Constructs a proof line made of a formula and a rule.
     *
     * @param   arguments    list of arguments
     * @throws  ArgumentException    if there are not exactly two
     *          arguments that are instances of
     *          {@link com.meyling.principia.logic.basic.Formula} and
     *          {@link Rule}.
     */
    public ProofLine(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (arguments.length != 2) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.OPERATOR_WITH_TWO_ARGUMENTS);
        }
        if (!(arguments[0] instanceof Formula)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.FIRST_ARGUMENT_TYPE
                + ParagraphConstants.FORMULA,
                arguments[0]);
        }
        if (arguments[0].containsPatternVariables()) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.FIRST_ARGUMENT
                + ParagraphConstants.MUST_NOT_CONTAIN_PATTERN_VARIABLE,
                arguments[0]);
        }
        if (!(arguments[1] instanceof Rule)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.SECOND_ARGUMENT_TYPE
                + ParagraphConstants.RULE,
                arguments[1]);
        }
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new ProofLine(arguments);
    }



    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer();
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append(", ");
            };
            buffer.append(getArgument(i));
        }
        return buffer.toString();
    };

}
