// $RCSfile: ProofLineList.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.paragraph;


import com.meyling.principia.argument.*;
import com.meyling.principia.module.*;
import com.meyling.principia.logic.basic.*;


/**
 * List of proof lines.
 *
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public class ProofLineList
        extends AbstractDynamicArgumentList implements Argument {


    /**
     * Constructs a list of proof lines.
     *
     * @param   arguments     list of proof lines
     * @throws  ArgumentException    if there is any argument
     *          that is not an instance of {@link ProofLine}
     */
    public ProofLineList(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        for (int i = 0; i < arguments.length; i++) {
            if (!(arguments[i] instanceof ProofLine)) {
                throw new ArgumentException(
                    ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                    ModuleCreator.getName(ProofLine.class)
                    + ParagraphConstants.EXPECTED_IN_ARGUMENT
                    + (i + 1),
                    arguments[i]);
            }
        }

    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new ProofLineList(arguments);
    }


    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("Prooflines: \n");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append("\n");
            };
            buffer.append(i + ":\t");
            buffer.append(getArgument(i));
        }
        return buffer.toString();
    };


}
