// $RCSfile: RuleDeclaration.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.paragraph;


import com.meyling.principia.argument.*;
import com.meyling.principia.module.*;
import com.meyling.principia.logic.basic.*;
import com.meyling.principia.logic.rule.*;

import java.util.*;


/**
 * Declares a logical rule.
 *
 * @version $Revision: 1.4 $
 * @author  Michael Meyling
 */
public class RuleDeclaration extends AbstractDynamicArgumentList
        implements Argument, ParagraphCheck {


    /**
     * Constructs a mathematical RuleDeclaration.
     *
     * @param   arguments     name of rule, its description and arguments
     * @throws  ArgumentException    if there are not at leas two
     *          arguments or the first argument is not an instance
     *          of {@link Text} or the second argument
     *          is no instance of {@link Text}.
     */
    public RuleDeclaration(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);

        if (arguments.length < 2) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.OPERATOR_WITH_AT_LEAST_TWO_ARGUMENTS);
        }
        if (!(arguments[0] instanceof Text)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.FIRST_ARGUMENT_TYPE
                + ParagraphConstants.QUOTED_STRING,
                arguments[0]);
        }
        if (!(arguments[1] instanceof Text)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.SECOND_ARGUMENT_TYPE
                + ParagraphConstants.QUOTED_STRING,
                arguments[1]);
        }
    }


    /**
     * Check if declaration is correct.
     *
     * @param   module   module
     * @param   label    label of paragraph
     * @throws  ArgumentException    if proof is not correct
     */
    public final void check(final Module module, final String label)
            throws ArgumentException {
        try {
            module.addRuleDeclaration(label, this, module);
        } catch (IllegalArgumentException e) {
            throw new ArgumentException(
                ParagraphConstants.CODE_METHOD_ARGUMENT_CHECK_FAILED,
                e.getMessage(),
                this.getArgument(0));
        }

    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new RuleDeclaration(arguments);
    }


    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("RuleDeclaration ");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append(", ");
            };
            buffer.append(getArgument(i));
        }
        return buffer.toString();
    }

}
