// $RCSfile: Sentence.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.logic.paragraph;


import com.meyling.principia.argument.*;
import com.meyling.principia.module.*;
import com.meyling.principia.logic.basic.*;


/**
 * Logical Sentence.
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public class Sentence extends AbstractArgumentList
        implements Argument {


    /**
     * Constructs a mathematical sentence.
     *
     * @param   arguments     sentence formula
     * @throws  ArgumentException    if there is not exactly one
     *          argument or the first argument is not an instance
     *          of {@link Formula} or it contains any
     *          {@link PatternVariable}.
     */
    public Sentence(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);

        if (arguments.length != 1) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.OPERATOR_WITH_ONE_ARGUMENT);
        }
        if (!(arguments[0] instanceof Formula)) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ParagraphConstants.FIRST_ARGUMENT_TYPE
                + ParagraphConstants.FORMULA,
                arguments[0]);
        }
        if (arguments[0].containsPatternVariables()) {
            throw new ArgumentException(
                ParagraphConstants.CODE_CONSTRUCTOR_ARGUMENT_CHECK_FAILED,
                ParagraphConstants.NO_PATTERN_VARIABLE_IN_PROPOSITION,
                arguments[0]);
        }
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new Sentence(arguments);
    }


    /** @inherited */
    public final String toString() {
        return "Sentence " + getArgument(0).toString();
    }

}
