// $RCSfile: Header.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.module;


import com.meyling.principia.argument.*;
import com.meyling.principia.logic.basic.*;


/**
 * header of a module.
 *
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public class Header extends AbstractArgumentList
        implements Argument {

    /**
     * Constructs a module header.
     *
     * @param   arguments    list of arguments
     * @throws  ArgumentException    if the argument list
     *          hasn't the length 5 or isn't made of
     *          {@link Specification}, {@link Headline}, 
     *          {@link Description}, {@link Email}
     *          and {@link AuthorList} in exactly that order.
     */
    public Header(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (arguments.length != 5) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.OPERATOR_WITH_FIVE_ARGUMENTS);
        }
        if (!(arguments[0] instanceof Specification)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.FIRST_ARGUMENT_TYPE
                + ModuleCreator.getName(Specification.class),
                arguments[0]);
        }
        if (!(arguments[1] instanceof Headline)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.SECOND_ARGUMENT_TYPE
                + ModuleCreator.getName(Headline.class),
                arguments[1]);
        }
        if (!(arguments[2] instanceof Description)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.THIRD_ARGUMENT_TYPE
                + ModuleCreator.getName(Description.class),
                arguments[2]);
        }
        if (!(arguments[3] instanceof Email)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.FOURTH_ARGUMENT_TYPE
                + ModuleCreator.getName(Email.class),
                arguments[3]);
        }
        if (!(arguments[4] instanceof AuthorList)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.FIFTH_ARGUMENT_TYPE
                + ModuleCreator.getName(AuthorList.class),
                arguments[4]);
        }
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments) 
            throws ArgumentException {
        return new Header(arguments);
    }



    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("Header\n");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append(", ");
            };
            buffer.append(getArgument(i));
        }
        return buffer.toString();
    };

}
