// $RCSfile: Import.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.module;


import com.meyling.principia.argument.*;
import com.meyling.principia.logic.basic.*;
import com.meyling.principia.logic.paragraph.*;


/**
 * Describes an import of a module.
 *
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public class Import extends AbstractArgumentList
        implements Argument {

    /**
     * Constructs an import description.
     *
     * @param   arguments    list of arguments
     * @throws  ArgumentException    if the argument list
     *          contains more than two or no entries or
     *          the arguments are not instances of
     *          {@link Specification} and
     *          {@link Label} (in this order).
     */
    public Import(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (arguments.length < 1 || arguments.length > 2) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.OPERATOR_WITH_ONE_TO_TWO_ARGUMENTS);
        }
        if (!(arguments[0] instanceof Specification)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.FIRST_ARGUMENT_TYPE
                + ModuleCreator.getName(Specification.class),
                arguments[0]);
        }
        if (arguments.length > 1 && !(arguments[1] instanceof LinkLabel)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.SECOND_ARGUMENT_TYPE
                + ModuleCreator.getName(LinkLabel.class),
                arguments[1]);
        }
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new Import(arguments);
    }



    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("Import ");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append(", ");
            };
            buffer.append(getArgument(i));
        }
        return buffer.toString();
    };

}
