// $RCSfile: LocationList.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.module;


import com.meyling.principia.argument.*;
import com.meyling.principia.logic.basic.*;


/**
 * List of locations where a module could be found.
 *
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public class LocationList
        extends AbstractDynamicArgumentList implements Argument {


    /**
     * Constructs a list of locations.
     *
     * @param   arguments     list of locations
     * @throws  ArgumentException    if there is any argument
     *          that is not an instance of {@link Location} or
     *          there isn't at least one entry.
     */
    public LocationList(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (arguments.length == 0) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(Location.class)
                + ModuleConstants.AT_LEAST_ONE_ARGUMENT_EXPECTED);
        }
        for (int i = 0; i < arguments.length; i++) {
            if (!(arguments[i] instanceof Location)) {
                throw new ArgumentException(
                    ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                    ModuleCreator.getName(Location.class)
                    + ModuleConstants.EXPECTED_IN_ARGUMENT
                    + (i + 1),
                    arguments[i]);
            }
        }

    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new LocationList(arguments);
    }


    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("List of locations: \n");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append("\n");
            };
            buffer.append(i + ":\t");
            buffer.append(getArgument(i));
        }
        return buffer.toString();
    };


}
