// $RCSfile: ParagraphList.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.module;


import com.meyling.principia.argument.*;
import com.meyling.principia.logic.basic.*;
import com.meyling.principia.logic.paragraph.*;


/**
 * List of {@link Paragraph}s
 *
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public class ParagraphList
        extends AbstractDynamicArgumentList implements Argument {


    /**
     * Constructs a list of paragraphs.
     *
     * @param   arguments     list of paragraphs
     * @throws  ArgumentException    if there is any argument
     *          that is not an instance of {@link Paragraph}
     */
    public ParagraphList(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        for (int i = 0; i < arguments.length; i++) {
            if (!(arguments[i] instanceof Paragraph)) {
                throw new ArgumentException(
                    ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                    ModuleCreator.getName(Paragraph.class)
                    + ModuleConstants.EXPECTED_IN_ARGUMENT
                    + (i + 1),
                    arguments[i]);
            }
        }

    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new ParagraphList(arguments);
    }


    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("Paragraphs: \n");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append("\n");
            };
            buffer.append(i + ":\t");
            buffer.append(getArgument(i));
        }
        return buffer.toString();
    };


}
