// $RCSfile: SimpleModuleCreator.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.module;


import com.meyling.principia.argument.*;
import com.meyling.principia.io.*;

import java.io.Reader;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;


/**
 * This class provides static methods for reading and
 * writing an {@link Argument} of the module package.
 *
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public final class SimpleModuleCreator {


    /**
     * for converting a <code>String</code> into an
     * (Argument) <ccode>class</code>
     */
    private final static Map string2class = ModuleCreator.getMap();

    /**
     * for converting a (Argument) <code>class</code> into an
     * <ccode>String</code>
     */
    private final static Map class2string = new HashMap();


    /**
     * Make attribut initialization.
     */
    static {
        // build inverse mapping
        for (Iterator it = string2class.entrySet().iterator(); it.hasNext(); ) {
            Map.Entry entry = (Map.Entry) it.next();
            class2string.put(entry.getValue(), entry.getKey());
        }

    }


    /**
     * Get the mapping of Strings to operator classes. The returned Map
     * should not be modified.
     *
     * @return  mapping of known Strings, do not change this Map
     */
    public final static Map getMap() {
        return string2class;
    }


    /**
     * Get name of known operator class.
     *
     * @param   c   look for name of this class
     * @return  name of operator class
     * @throws  IllegalArgumentException    if operator class <code>c</code>
     *          is unknown
     */
    public final static String getName(final Class c)
            throws IllegalArgumentException {
        final String result;
        try {
            result = (String) class2string.get(c);
        } catch (Exception e) {
            throw new IllegalArgumentException(e.toString());
        }
        if (result == null) {
            throw new IllegalArgumentException(c.getName()
                + ModuleConstants.UNKNOWN_CLASS);
        }
        return result;
    }


    /**
     * Create an {@link Argument} out of an {@link TextInput} object.
     *
     * @param   input   source to read from
     * @param   parsing maps {@link Argument}s into {@link Position}s
     * @return  new generated Argument
     * @throws  ParsingException    if creation failed
     */
    public final static Argument readArgument(final TextInput input, 
            final ParsingTable parsing)
            throws ParsingException {
        return ArgumentCreator.readArgument(input, string2class, parsing);
    }


    /**
     * Write an {@link Argument} to an {@link Output} object.
     *
     * @param   output   destination to write to
     * @param   argument this object will be written
     * @throws  IllegalArgumentException    if writing failed
     */
    public final static void writeArgument(final Output output,
            final Argument argument) throws IllegalArgumentException {
        ArgumentCreator.writeArgument(output, argument, class2string);
    }


    /**
     * Write an {@link Argument} to an <code>String</code>.
     *
     * @param   argument this object will be written
     * @return  created string
     * @throws  IllegalArgumentException    if writing failed
     */
    public final static String writeArgument(
            final Argument argument) throws IllegalArgumentException {
        return ArgumentCreator.writeArgument(argument, class2string);
    }


}
