// $RCSfile: Specification.java,v $
//
// Project "Principia"
//
// Michael Meyling
// Stoltenstrasse 38
// 22119 Hamburg
// Germany

package com.meyling.principia.module;


import com.meyling.principia.argument.*;
import com.meyling.principia.logic.basic.*;


/**
 * Describes a specificiation of a module, that means
 * its name, versions and possible &quot;physical&quot;
 * locations.
 *
 * @version $Revision: 1.2 $
 * @author  Michael Meyling
 */
public class Specification extends AbstractArgumentList
        implements Argument {

    /**
     * Constructs a module specification.
     *
     * @param   arguments    list of arguments
     * @throws  ArgumentException    if the argument list
     *          hasn't the length 4 or isn't made of
     *          {@link Name}, {@link Version}, {@link Version}
     *          and {@link LocationList} in exactly that order.
     */
    public Specification(final Argument[] arguments)
            throws ArgumentException {
        super(arguments);
        if (arguments.length != 4) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_NUMBER,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.OPERATOR_WITH_FOUR_ARGUMENTS);
        }
        if (!(arguments[0] instanceof Name)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.FIRST_ARGUMENT_TYPE
                + ModuleCreator.getName(Name.class),
                arguments[0]);
        }
        if (!(arguments[1] instanceof Version)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.SECOND_ARGUMENT_TYPE
                + ModuleCreator.getName(Version.class),
                arguments[1]);
        }
        if (!(arguments[2] instanceof Version)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.THIRD_ARGUMENT_TYPE
                + ModuleCreator.getName(Version.class),
                arguments[2]);
        }
        if (!(arguments[3] instanceof LocationList)) {
            throw new ArgumentException(
                ModuleConstants.CODE_CONSTRUCTOR_WRONG_ARGUMENT_TYPE,
                ModuleCreator.getName(this.getClass())
                + ModuleConstants.FOURTH_ARGUMENT_TYPE
                + ModuleCreator.getName(LocationList.class),
                arguments[3]);
        }
    }


    /** @inherited */
    public final Argument create(final Argument[] arguments)
            throws ArgumentException {
        return new Specification(arguments);
    }



    /** @inherited */
    public final String toString() {
        StringBuffer buffer = new StringBuffer("Specification: ");
        for (int i = 0; i < getArgumentSize(); i++) {
            if (i != 0) {
                buffer.append(", ");
            };
            buffer.append(getArgument(i));
        }
        return buffer.toString();
    };

}
